////////////////////////////////////////////////////////////////
// ライフゲームガジェット
//   Copyright (C) 2008 - 2012 AAOh inc.
////////////////////////////////////////////////////////////////

/**
 * @fileOverview ライフゲームガジェット
 * @author       $Author: fujishiro $
 */

(function () {
/**
 * Main。
 *
 * @constructor
 */
lifegame.Main = aaoh.kaze.Gadget.extend(
	aaoh.sora.MainEx, function() {
	this.OPACITY_PTN_SET_BG = 40;

	this.ICON_PTN_PREFIX = 'images/icon-ptn';
	this.ICON_PTN_SUFFIX = '.png';

	this.MODE_ANI_1     = 1;
	this.MODE_ANI_2     = 2;
	this.MODE_ANI_CLEAR = 9;

	this.cellRowSize = 0;
	this.cellColSize = 0;
	this.cellVals    = new Array();

	this.bgOnColor   = '#7f7f7f';
	this.bgOffColor  = '#ffffff';
	this.borderColor = '#b4b4b4';
	this.borderImg   = 'sub-images/line-bottom.png';

	this.lblGens  = this.getLblGens();
	this.genCnt   = 0;
	this.genTimer = new aaoh.kaze.Timer(this);

	this.flgPtnSet = false;
	this.ptnDefNum = parseInt(aaoh.sora.App.model.data.ptnDef, 10);

	this.flgOnDock = aaoh.sora.App.control.flgOnDock;
	//this.dispTimer = new aaoh.kaze.Timer(this);

	var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm');
	this.imgPtnAni = new aaoh.kaze.animation.FadeinAnimation(imgPtnNm);
	this.fadeinAni = null;
	this.fadeoutAn = null;
	if (!aaoh.kaze.Type.isIE() && !aaoh.kaze.Type.isYahoo()) {
		this.fadeinAni  = new aaoh.kaze.animation.FadeinAnimation(imgPtnNm);
		this.fadeoutAni = new aaoh.kaze.animation.FadeoutAnimation(imgPtnNm);
		this.fadeoutAni.startCallable = function(element, startValue) {
			aaoh.kaze.Element.setDisplay(element, true);
			aaoh.kaze.Element.setOpacity(element, startValue);
		}
		this.fadeoutAni.stopCallable = function(element, stopValue) {
			aaoh.kaze.Element.setDisplay(element, false);
			aaoh.kaze.Element.setOpacity(element, stopValue);
		}
	}

	this.cellAniTimer1 = new aaoh.kaze.Timer(this);
	this.cellAniTimer2 = new aaoh.kaze.Timer(this);
	this.cellAniTimer3 = new aaoh.kaze.Timer(this);
	this.cellAniTime1  = 1;
	this.cellAniTime2  = 200;
	this.cellAniTime3  = 200;
});
	/**
	 * セルの取得。
	 *
	 * @param  {Number} row 行
	 * @param  {Number} col 列
	 * @return {Object} セル
	 */
	lifegame.Main.prototype.
	getCell = function(row, col) {
		var cell = null;

		var rowStr = null;
		if (row < 9) {
			rowStr = '0' + (row + 1);
		} else {
			rowStr = ''  + (row + 1);
		}
		var colStr = null;
		if (col < 9) {
			colStr = '0' + (col + 1);
		} else {
			colStr = ''  + (col + 1);
		}

		if (this.flgOnDock) {
			cell = aaoh.kaze.Gadget.$('cell_' + rowStr + '_' + colStr);

		} else {
			cell = aaoh.kaze.Gadget.$('cell_' + rowStr + '_' + colStr + '_u');
		}

		return cell;
	}

	/**
	 * セルリストの取得。
	 *
	 * @return {Array} セルリスト
	 */
	lifegame.Main.prototype.
	getCells = function() {
		var cells = null;

		if (this.flgOnDock) {
			cells = this.getOnDockCells();

		} else {
			cells = this.getUnDockCells();
		}

		return cells;
	}

	/**
	 * オンドックのセルリストの取得。
	 *
	 * @return {Array} セルリスト
	 */
	lifegame.Main.prototype.
	getOnDockCells = function() {
		var cells = new Array();

		for (
			var row = 0;
			row < aaoh.sora.App.control.ONDOCK_CELL_ROW_SIZE;
			row++
		) {
			var rowStr = null;
			if (row < 9) {
				rowStr = '0' + (row + 1);
			} else {
				rowStr = ''  + (row + 1);
			}

			for (
				var col = 0;
				col < aaoh.sora.App.control.ONDOCK_CELL_COL_SIZE;
				col++
			) {
				var colStr = null;
				if (col < 9) {
					colStr = '0' + (col + 1);
				} else {
					colStr = ''  + (col + 1);
				}

				var cell = aaoh.kaze.Gadget.$('cell_' + rowStr + '_' + colStr);
				if (cell == null) {
					continue;
				}

				cells.push(cell);
			}
		} // for (var row = 0; ...)

		return cells;
	}

	/**
	 * アンドックのセルリストの取得。
	 *
	 * @return {Array} セルリスト
	 */
	lifegame.Main.prototype.
	getUnDockCells = function() {
		var cells = new Array();

		for (
			var row = 0;
			row < aaoh.sora.App.control.UNDOCK_CELL_ROW_SIZE;
			row++
		) {
			var rowStr = null;
			if (row < 9) {
				rowStr = '0' + (row + 1);
			} else {
				rowStr = ''  + (row + 1);
			}

			for (
				var col = 0;
				col < aaoh.sora.App.control.UNDOCK_CELL_COL_SIZE;
				col++
			) {
				var colStr = null;
				if (col < 9) {
					colStr = '0' + (col + 1);
				} else {
					colStr = ''  + (col + 1);
				}

				var cell = aaoh.kaze.Gadget.$('cell_' + rowStr + '_' + colStr + '_u');
				if (cell == null) {
					continue;
				}

				cells.push(cell);
			}
		} // for (var row = 0; ...)

		return cells;
	}

	/**
	 * パターンボタンリストの取得。
	 *
	 * @return {Array} パターンボタンリスト
	 */
	lifegame.Main.prototype.
	getPtnButtons = function() {
		var btnPtn   = null;
		var btnPtn_d = null;

		if (this.flgOnDock) {
			btnPtn   = aaoh.kaze.Gadget.$('btnPtn');
			btnPtn_d = aaoh.kaze.Gadget.$('btnPtn_d');

		} else {
			btnPtn   = aaoh.kaze.Gadget.$('btnPtn_u');
			btnPtn_d = aaoh.kaze.Gadget.$('btnPtn_d_u');
		}

		return new Array(btnPtn, btnPtn_d);
	}

	/**
	 * 開始ボタンリストの取得。
	 *
	 * @return {Array} 開始ボタンリスト
	 */
	lifegame.Main.prototype.
	getStartButtons = function() {
		var btnStart = null;
		var btnStop  = null;

		if (this.flgOnDock) {
			btnStart = aaoh.kaze.Gadget.$('btnStart');
			btnStop  = aaoh.kaze.Gadget.$('btnStop');

		} else {
			btnStart = aaoh.kaze.Gadget.$('btnStart_u');
			btnStop  = aaoh.kaze.Gadget.$('btnStop_u');
		}

		return new Array(btnStart, btnStop);
	}

	/**
	 * クリアボタンリストの取得。
	 *
	 * @return {Array} クリアボタンリスト
	 */
	lifegame.Main.prototype.
	getClearButtons = function() {
		var btnClear   = null;
		var btnClear_d = null;

		if (this.flgOnDock) {
			btnClear   = aaoh.kaze.Gadget.$('btnClear');
			btnClear_d = aaoh.kaze.Gadget.$('btnClear_d');

		} else {
			btnClear   = aaoh.kaze.Gadget.$('btnClear_u');
			btnClear_d = aaoh.kaze.Gadget.$('btnClear_d_u');
		}

		return new Array(btnClear, btnClear_d);
	}

	/**
	 * 世代ラベルリストの取得。
	 *
	 * @return {Array} 世代ラベルリスト
	 */
	lifegame.Main.prototype.
	getLblGens = function() {
		var lblGenTtl = null;
		var lblGenCnt = null;

		if (this.flgOnDock) {
			lblGenTtl = aaoh.kaze.Gadget.$('lblGenTtl');
			lblGenCnt = aaoh.kaze.Gadget.$('lblGenCnt');

		} else {
			lblGenTtl = aaoh.kaze.Gadget.$('lblGenTtl_u');
			lblGenCnt = aaoh.kaze.Gadget.$('lblGenCnt_u');
		}

		return new Array(lblGenTtl, lblGenCnt);
	}

	/**
	 * ステータス欄の取得。
	 *
	 * @return {Object} ステータス欄
	 */
	lifegame.Main.prototype.
	getStatus = function() {
		var status = null;

		if (this.flgOnDock) {
			status = aaoh.kaze.Gadget.$('status');

		} else {
			status = aaoh.kaze.Gadget.$('status_u');
		}

		return status;
	}

	/**
	 * パターン設定の取得。
	 *
	 * @return {Object} パターン設定
	 */
	lifegame.Main.prototype.
	getPtnSetBg = function() {
		var ptnSetBg = null;

		if (this.flgOnDock) {
			ptnSetBg = aaoh.kaze.Gadget.$('ptnSetBg');

		} else {
			ptnSetBg = aaoh.kaze.Gadget.$('ptnSetBg_u');
		}

		return ptnSetBg;
	}

	/**
	 * パターン設定の取得。
	 *
	 * @return {Object} パターン設定
	 */
	lifegame.Main.prototype.
	getPtnSet = function() {
		var ptnSet = null;

		if (this.flgOnDock) {
			ptnSet = aaoh.kaze.Gadget.$('ptnSet');

		} else {
			ptnSet = aaoh.kaze.Gadget.$('ptnSet_u');
		}

		return ptnSet;
	}

	/**
	 * パターン設定左ボタンの取得。
	 *
	 * @return {Object} パターン設定左ボタン
	 */
	lifegame.Main.prototype.
	getPtnLeftButton = function() {
		var btnPtnLeft = null;

		if (this.flgOnDock) {
			btnPtnLeft = aaoh.kaze.Gadget.$('btnPtnLeft');

		} else {
			btnPtnLeft = aaoh.kaze.Gadget.$('btnPtnLeft_u');
		}

		return btnPtnLeft;
	}

	/**
	 * パターン設定右ボタンの取得。
	 *
	 * @return {Object} パターン設定右ボタン
	 */
	lifegame.Main.prototype.
	getPtnRightButton = function() {
		var btnPtnRight = null;

		if (this.flgOnDock) {
			btnPtnRight = aaoh.kaze.Gadget.$('btnPtnRight');

		} else {
			btnPtnRight = aaoh.kaze.Gadget.$('btnPtnRight_u');
		}

		return btnPtnRight;
	}

	/**
	 * パターン設定名前画像の取得。
	 *
	 * @return {Object} パターン設定名前画像
	 */
	lifegame.Main.prototype.
	getPtnImageNm = function() {
		var imgPtnNm = null;

		if (this.flgOnDock) {
			imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm');

		} else {
			imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm_u');
		}

		return imgPtnNm;
	}

	/**
	 * セル番号の取得。
	 *
	 * @param  {String} id セルID
	 * @return {Number}    セル番号
	 */
	lifegame.Main.prototype.
	getCellNo = function(id) {
		if (id == null || id.length < 10) {
			return 0;
		}

		var strRow = id.substring(5, 7);
		var strCol = id.substring(8, 10);
		var rowNo  = parseInt(strRow, 10) - 1;
		var colNo  = parseInt(strCol, 10) - 1;

		return (rowNo * this.cellColSize) + colNo;
	}

	/**
	 * 周囲のセル値の取得。
	 *
	 * @param  {Number} cellNo セル番号
	 * @return {Array}         周囲のセル値
	 */
	lifegame.Main.prototype.
	getAroundCellVals = function(cellNo) {
		var cellSize = this.cellRowSize * this.cellColSize;
		var rowNo    = parseInt(cellNo / this.cellColSize, 10);
		var colNo    = cellNo % this.cellColSize;

		// 周囲のセルの範囲取得
		var rangeFm1 = (rowNo - 1) * this.cellColSize;
		if (rangeFm1 < 0) {
			rangeFm1 = cellSize + rangeFm1;
		}
		var rangeTo1 = rangeFm1 + this.cellColSize;
		var rangeFm2 = (rowNo + 0) * this.cellColSize;
		var rangeTo2 = rangeFm2 + this.cellColSize;
		var rangeFm3 = (rowNo + 1) * this.cellColSize;
		if (rangeFm3 >= cellSize) {
			rangeFm3 = cellSize - rangeFm3;
		}
		var rangeTo3 = rangeFm3 + this.cellColSize;

		// 周囲のセル番号の取得
		var val11 = rangeFm1 + colNo - 1;
		var val12 = rangeFm1 + colNo + 0;
		var val13 = rangeFm1 + colNo + 1;
		var val21 = rangeFm2 + colNo - 1;
		var val23 = rangeFm2 + colNo + 1;
		var val31 = rangeFm3 + colNo - 1;
		var val32 = rangeFm3 + colNo + 0;
		var val33 = rangeFm3 + colNo + 1;
		if (val11 < rangeFm1) {
			val11 = rangeTo1 - 1;
		}
		if (rangeTo1 <= val13) {
			val13 = rangeFm1 + 0;
		}
		if (val21 < rangeFm2) {
			val21 = rangeTo2 - 1;
		}
		if (rangeTo2 <= val23) {
			val23 = rangeFm2 + 0;
		}
		if (val31 < rangeFm3) {
			val31 = rangeTo3 - 1;
		}
		if (rangeTo3 <= val33) {
			val33 = rangeFm3 + 0;
		}

		return new Array(
			this.cellVals[val11], this.cellVals[val12], this.cellVals[val13],
			this.cellVals[val21],                       this.cellVals[val23],
			this.cellVals[val31], this.cellVals[val32], this.cellVals[val33]
		);
	}

	/**
	 * トグルボタン設定。
	 *
	 * @param {Object} showElement 表示オブジェクト
	 * @param {Object} hideElement 非表示オブジェクト
	 */
	lifegame.Main.prototype.
	setToggleButton = function(showElement, hideElement) {
		if (showElement != null) {
			aaoh.kaze.Element.setDisplay(showElement, true);
		}

		if (hideElement != null) {
			aaoh.kaze.Element.setDisplay(hideElement, false);
		}
	}

	/**
	 * トグルボタン設定。
	 *
	 * @param {Object} showElement 表示オブジェクト
	 * @param {Object} hideElement 非表示オブジェクト
	 */
	lifegame.Main.prototype.
	setFadeToggleButton = function(showElement, hideElement) {
		if (this.fadeinAni == null || this.fadeoutAni == null) {
			return this.setToggleButton(showElement, hideElement);
		}

		if (showElement != null) {
			aaoh.kaze.Element.setDisplay(showElement, true);
			this.fadeinAni.element = showElement;
			this.fadeinAni.start();
		}

		if (hideElement != null) {
			this.fadeoutAni.element = hideElement;
			this.fadeoutAni.start();
		}
	}

	/**
	 * トグルセル設定。
	 *
	 * @param {Object} cell 表示オブジェクト
	 */
	lifegame.Main.prototype.
	setToggleCell = function(cell) {
		if (cell == null) {
			return;
		}
		var flgGoogle = aaoh.kaze.Type.isGoogle();
		var cellId    = null;

		// google対応
		if (flgGoogle) {
			cellId = cell.name;
			cell   = cell.parentElement;

		} else {
			cellId = cell.id;
		}

		var cellNo = this.getCellNo(cellId);
		if (this.cellVals[cellNo]) {
			aaoh.kaze.Element.setBackground(cell, this.bgOffColor);

		} else {
			aaoh.kaze.Element.setBackground(cell, this.bgOnColor);
		}

		// セル値の反転
		this.cellVals[cellNo] = !this.cellVals[cellNo];

		this.genCnt = 1;
	}

	/**
	 * セルのクリア。
	 *
	 * @param {Object} cells 表示オブジェクト
	 */
	lifegame.Main.prototype.
	clearCells = function(cells) {
		if (cells == null) {
			return;
		}
		var flgGoogle = aaoh.kaze.Type.isGoogle();

		for (var cnt = 0; cnt < cells.length; cnt++) {
			var cell = cells[cnt];

			// google対応
			if (flgGoogle) {
				cell = cell.parentElement;
			}

			aaoh.kaze.Element.setBackground(cell, this.bgOffColor);
		}

		// セル値の初期化
		var cellSize = this.cellRowSize * this.cellColSize;
		this.cellVals = new Array(cellSize);
		for (var cnt = 0; cnt < cellSize; cnt++) {
			this.cellVals[cnt] = false;
		}

		for (var cnt = 0; cnt < this.lblGens.length; cnt++) {
			var lblGen = this.lblGens[cnt];
			aaoh.kaze.Element.setVisible(lblGen, false);
		}

		this.genCnt = 1;
	}

	/**
	 * 開始処理。
	 */
	lifegame.Main.prototype.
	doStart = function() {
		var btnPtns = this.getPtnButtons();
		this.setToggleButton(btnPtns[1], btnPtns[0]);

		var btnStarts = this.getStartButtons();
		this.setToggleButton(btnStarts[1], btnStarts[0]);

		var btnClears = this.getClearButtons();
		this.setToggleButton(btnClears[1], btnClears[0]);

		for (var cnt = 0; cnt < this.lblGens.length; cnt++) {
			var lblGen = this.lblGens[cnt];
			aaoh.kaze.Element.setVisible(lblGen, true);
		}

		if (this.genCnt <= 0) {
			this.genCnt = 1;
		}

		// 開始アニメーション
		this.cellAniTimer1.setTimeout(function() {
			var cells = this.getCells();
			this.doCellAnimetion(cells, this.MODE_ANI_1);

			this.cellAniTimer2.setTimeout(function() {
				this.doCellAnimetion(cells, this.MODE_ANI_2);

				this.cellAniTimer3.setTimeout(function() {
					this.doCellAnimetion(cells, this.MODE_ANI_CLEAR);

					// 判定処理
					this.doAlive();
					this.genTimer.setInterval(function() {
							aaoh.sora.App.main.doAlive();
						}, aaoh.sora.App.model.data.numInterval
					);
				}, this.cellAniTime3);
			}, this.cellAniTime2);
		}, this.cellAniTime1);
	}

	/**
	 * 停止処理。
	 *
	 * @param {Boolean} flgStop 全滅フラグ
	 */
	lifegame.Main.prototype.
	doStop = function(flgStop) {
		this.genTimer.clearInterval();

		if (this.genCnt <= 1) {
			// 停止処理
			var btnPtns = this.getPtnButtons();
			this.setToggleButton(btnPtns[0], btnPtns[1]);

			var btnStarts = this.getStartButtons();
			this.setToggleButton(btnStarts[0], btnStarts[1]);

			var btnClears = this.getClearButtons();
			this.setToggleButton(btnClears[0], btnClears[1]);
			return;
		}

		if (flgStop) {
			this.genCnt = 1;
		}

		// 停止アニメーション
		this.cellAniTimer1.setTimeout(function() {
			var cells = this.getCells();
			this.doCellAnimetion(cells, this.MODE_ANI_2);

			this.cellAniTimer2.setTimeout(function() {
				this.doCellAnimetion(cells, this.MODE_ANI_1);

				this.cellAniTimer3.setTimeout(function() {
					this.doCellAnimetion(cells, this.MODE_ANI_CLEAR);

					// 停止処理
					var btnPtns = this.getPtnButtons();
					this.setToggleButton(btnPtns[0], btnPtns[1]);

					var btnStarts = this.getStartButtons();
					this.setToggleButton(btnStarts[0], btnStarts[1]);

					var btnClears = this.getClearButtons();
					this.setToggleButton(btnClears[0], btnClears[1]);
				}, this.cellAniTime3);
			}, this.cellAniTime2);
		}, this.cellAniTime1);
	}

	/**
	 * 判定処理。
	 */
	lifegame.Main.prototype.
	doAlive = function() {
		var flgNextGen = false;
		var flgGoogle  = aaoh.kaze.Type.isGoogle();

		var cells        = this.getCells();
		var nextCellVals = new Array(this.cellRowSize * this.cellColSize);
		for (var cnt = 0; cnt < cells.length; cnt++) {
			var cell    = cells[cnt];
			var cellVal = this.cellVals[cnt];

			// google対応
			if (flgGoogle) {
				cell = cell.parentElement;
			}

			// 周囲のセル値の取得
			var aroundCellVals = this.getAroundCellVals(cnt);

			// 生存判定
			var flgAlive = this.isAlive(cellVal, aroundCellVals);
			if (flgAlive) {
				if (!flgNextGen) {
					flgNextGen = true;
				}
				aaoh.kaze.Element.setBackground(cell, this.bgOnColor);
				nextCellVals[cnt] = true;

			} else {
				aaoh.kaze.Element.setBackground(cell, this.bgOffColor);
				nextCellVals[cnt] = false;
			}
		} // for (var cnt = 0; cnt < cells.length; cnt++)
		this.cellVals = nextCellVals;

		// 生存世代表示
		aaoh.kaze.Element.setValue(this.lblGens[1], this.genCnt);

		if (flgNextGen) {
			// 切番チェック
			if (this.genCnt > 0 && this.genCnt % 100 == 0) {
				this.cellAniTimer1.setTimeout(function() {
					this.doCellAnimetion(cells, this.MODE_ANI_1);
				}, this.cellAniTime1);
			}

			this.genCnt++;
			if (this.genCnt > aaoh.sora.App.control.MAX_GEN_CNT) {
				this.genCnt = 1;
			}

		} else {
			// 全滅したら止める
			this.doStop(true);
		}
	}

	/**
	 * 生存判定。
	 *
	 * @param  {Boolean} cellVal        セル値
	 * @param  {Array}   aroundCellVals 周囲のセル値
	 * @return {Boolean} 生存であればtrueを返す
	 */
	lifegame.Main.prototype.
	isAlive = function(cellVal, aroundCellVals) {
		if (aroundCellVals == null) {
			return false;
		}
		var flgAlive = false;

		var aroundAliveCnt = 0;
		for (var cnt = 0; cnt < aroundCellVals.length; cnt++) {
			if (aroundCellVals[cnt]) {
				aroundAliveCnt++;
			}
		}

		// 誕生: 死んでいるセル値の周囲のセル値が3つ生きているとき
		// 維持: 生きているセル値の周囲のセル値が2 or 3つ生きているとき
		// 死亡: それ以外のとき
		if (!cellVal && aroundAliveCnt == 3) {
			flgAlive = true;

		} else if (cellVal && (aroundAliveCnt == 2 || aroundAliveCnt == 3)) {
			flgAlive = true;
		}

		return flgAlive;
	}

	/**
	 * セルアニメーション。
	 *
	 * @param {Array}  cells セルオブジェクト
	 * @param {Number} mode  アニメーションモード
	 */
	lifegame.Main.prototype.
	doCellAnimetion = function(cells, mode) {
		if (cells == null) {
			return;
		}
		if (this.genCnt <= 0) {
			return;
		}

		var cellCount = 0;
		var flgGoogle = aaoh.kaze.Type.isGoogle();

		for (var row = 0; row < this.cellRowSize; row++) {
			for (var col = 0; col < this.cellColSize; col++) {
				var cell    = cells[cellCount];
				var cellVal = this.cellVals[cellCount];

				// google対応
				if (flgGoogle) {
					cell = cell.parentElement;
				}

				var cellColor = this.getCellAnimetionColor(
					row, col, cellCount, mode
				);
				aaoh.kaze.Element.setBackground(cell, cellColor);

				cellCount++;
			} // for (var col = 0; col < this.cellColSize; col++)
		} // for (var row = 0; row < this.cellRowlSize; row++)
	}

	/**
	 * セルアニメーション。
	 *
	 * @param {Number} row   セル行
	 * @param {Number} col   セル列
	 * @param {Number} count セル番号
	 * @param {Number} mode  アニメーションモード
	 */
	lifegame.Main.prototype.
	getCellAnimetionColor = function(row, col, count, mode) {
		var cellVal = this.cellVals[count];

		if (!cellVal) {
			switch (mode) {
				case this.MODE_ANI_1:
					if (row % 2 == 0) {
						if (col % 2 == 1) {
							cellVal = !cellVal;
						}
					} else {
						if (col % 2 == 0) {
							cellVal = !cellVal;
						}
					}
					break;

				case this.MODE_ANI_2:
					if (row % 2 == 0) {
						if (col % 2 == 0) {
							cellVal = !cellVal;
						}
					} else {
						if (col % 2 == 1) {
							cellVal = !cellVal;
						}
					}
					break;

				case this.MODE_ANI_CLEAR:
				default:
					break;
			} // switch (mode)
		} // if (!cellVal)

		var cellColor = null;
		if (cellVal) {
			cellColor = this.bgOnColor;

		} else {
			cellColor = this.bgOffColor;
		}
		return cellColor;
	}

	/**
	 * パターン設定。
	 *
	 * @param {Number} ptnNo パターン番号
	 */
	lifegame.Main.prototype.
	doPtnSet = function(ptnNo) {
		if (isNaN(ptnNo)) {
			return;
		}

		// パターン取得
		var rowPtnSet = aaoh.sora.App.control.PTN[ptnNo - 1];
		if (rowPtnSet == null) {
			return;
		}
		var flgGoogle = aaoh.kaze.Type.isGoogle();

		for (var rowCnt = 0; rowCnt < rowPtnSet.length; rowCnt++) {
			var colPtnSet = rowPtnSet[rowCnt];
			if (colPtnSet == null) {
				continue;
			}

			for (var colCnt = 0; colCnt < colPtnSet.length; colCnt++) {
				var cellPtnSet = colPtnSet[colCnt];
				if (!cellPtnSet) {
					continue;
				}

				// パターン設定
				var cell   = this.getCell(rowCnt, colCnt);
				var cellNo = rowCnt * this.cellColSize + colCnt;
				if (cell == null) {
					continue;
				}

				// google対応
				if (flgGoogle) {
					cell = cell.parentElement;
				}

				aaoh.kaze.Element.setBackground(cell, this.bgOnColor);
				this.cellVals[cellNo] = true;
			}
		} // for (var rowCnt = 0; rowCnt < rowPtnSet.length; rowCnt++)
	}

	/**
	 * 現在パターン番号取得。
	 *
	 * @param  {Object} imgPtnNm パターンイメージ
	 * @return {Number} パターン番号
	 */
	lifegame.Main.prototype.
	getNowPtnNo = function(imgPtnNm) {
		if (imgPtnNm == null) {
			return this.ptnDefNum;
		}

		var imgPtnStr   = aaoh.kaze.Element.getImage(imgPtnNm);
		var indexPrefix = imgPtnStr.indexOf(this.ICON_PTN_PREFIX);
		if (indexPrefix < 0) {
			return this.ptnDefNum;

		} else {
			indexPrefix = indexPrefix + this.ICON_PTN_PREFIX.length;
		}

		var indexSuffix = imgPtnStr.indexOf(this.ICON_PTN_SUFFIX, indexPrefix);
		if (indexSuffix < 0) {
			return this.ptnDefNum;
		}

		var ptnNoStr = imgPtnStr.substring(indexPrefix, indexSuffix);

		return parseInt(ptnNoStr, 10);
	}

	/**
	 * 次パターン番号取得。
	 *
	 * @param  {Object} imgPtnNm パターンイメージ
	 * @return {Number} パターン番号
	 */
	lifegame.Main.prototype.
	getNextPtnNo = function(imgPtnNm) {
		var ptnNo = this.getNowPtnNo(imgPtnNm) + 1;
		if (ptnNo > aaoh.sora.App.control.PTN.length) {
			return 1;
		}

		return ptnNo;
	}

	/**
	 * 前パターン番号取得。
	 *
	 * @param  {Object} imgPtnNm パターンイメージ
	 * @return {Number} パターン番号
	 */
	lifegame.Main.prototype.
	getPreviousPtnNo = function(imgPtnNm) {
		var ptnNo = this.getNowPtnNo(imgPtnNm) - 1;
		if (ptnNo <= 0) {
			return aaoh.sora.App.control.PTN.length;
		}

		return ptnNo;
	}

	/**
	 * パターンイメージの設定。
	 *
	 * @param {Object} imgPtnNm パターンイメージ
	 * @param {Number} ptnNo    パターン番号
	 */
	lifegame.Main.prototype.
	setImgPtnNm = function(imgPtnNm, ptnNo) {
		aaoh.kaze.Element.setImage(
			imgPtnNm,
			aaoh.sora.App.main.ICON_PTN_PREFIX
			+ ptnNo
			+ aaoh.sora.App.main.ICON_PTN_SUFFIX
		);

		var ttlPtn = aaoh.sora.App.control.locale.getString('TTL_PTN' + ptnNo);
		aaoh.kaze.Element.setTooltip(imgPtnNm, ttlPtn);

		this.imgPtnAni.element = imgPtnNm;
		this.imgPtnAni.start();
	}

	/**
	 * フォーカス設定。
	 */
	lifegame.Main.prototype.
	setFocus = function() {
		// 何もしない
	}

	/**
	 * 表示データの初期化。
	 */
	lifegame.Main.prototype.
	initData = function() {
		this.doStop(true);
		var ptnSet = this.getPtnSet();

		var flgOnDock = aaoh.sora.App.control.flgOnDock;
		var type      = aaoh.sora.App.model.data.type;

		this.flgOnDock = flgOnDock;
		this.ptnDefNum = parseInt(aaoh.sora.App.model.data.ptnDef, 10);

		this.initToolbarData(type);

		if (flgOnDock) {
			var main    = aaoh.kaze.Gadget.$('onDock');
			var srcMain = null;
			if (aaoh.kaze.Type.isSidebar()) {
				srcMain = 'lib/aaoh.sora.t.png';

			} else {
				srcMain = 'images/lifegame' + type + '-base.png';
			}
			// IE6では入れ子透過PNG対応しない
			aaoh.kaze.Element.setBackgroundImage(main, srcMain, false);

		} else {
			var main    = aaoh.kaze.Gadget.$('unDock');
			var srcMain = null;
			if (aaoh.kaze.Type.isSidebar()) {
				srcMain = 'lib/aaoh.sora.t.png';

			} else {
				srcMain = 'images/lifegame' + type + '-undock.png';
			}
			// IE6では入れ子透過PNG対応しない
			aaoh.kaze.Element.setBackgroundImage(main, srcMain, false);
		} // if (flgOnDock)

		// 透過PNG対応
		if (aaoh.kaze.Type.isSidebar()) {
			var sidebarImgBg = aaoh.kaze.Gadget.$('sidebarImgBg');

			var srcSidebarImgBg = null;
			if (flgOnDock) {
				srcSidebarImgBg = 'images/lifegame' + type + '-base_sidebar.png';
				sidebarImgBg.style.width
				= aaoh.sora.App.control.WINDOW_ONDOCK_WIDTH + 'px';
				sidebarImgBg.style.height
					= aaoh.sora.App.control.WINDOW_ONDOCK_HEIGHT + 'px';

			} else {
				srcSidebarImgBg = 'images/lifegame' + type + '-undock_sidebar.png';
				sidebarImgBg.style.width
				= aaoh.sora.App.control.WINDOW_UNDOCK_WIDTH + 'px';
				sidebarImgBg.style.height
					= aaoh.sora.App.control.WINDOW_UNDOCK_HEIGHT + 'px';
			}

			aaoh.kaze.Element.setImage(sidebarImgBg, srcSidebarImgBg);
		}

		// 色設定
		switch (type) {
			default:
			case '1':
				this.bgOnColor   = '#7f7f7f';
				this.bgOffColor  = '#ffffff';
				this.borderColor = '#b4b4b4';
				this.borderImg   = 'sub-images/line-bottom.png';
				break;

			case '2':
				this.bgOnColor   = '#ffffff';
				this.bgOffColor  = '#000000';
				this.borderColor = '#464646';
				this.borderImg   = 'sub-images/line-top.png';
				break;

			case '3':
				this.bgOnColor   = '#ffffff';
				this.bgOffColor  = '#ff0000';
				this.borderColor = '#ff8080';
				this.borderImg   = 'sub-images/line-top_red.png';
				break;
		}

		this.lblGens  = this.getLblGens();
		if (flgOnDock) {
			this.cellRowSize = aaoh.sora.App.control.ONDOCK_CELL_ROW_SIZE;
			this.cellColSize = aaoh.sora.App.control.ONDOCK_CELL_COL_SIZE;

			this.initOnDockCell(type);
			this.initOnDockData(type);

		} else {
			this.cellRowSize = aaoh.sora.App.control.UNDOCK_CELL_ROW_SIZE;
			this.cellColSize = aaoh.sora.App.control.UNDOCK_CELL_COL_SIZE;

			this.initUnDockCell(type);
			this.initUnDockData(type);
		}

		// デフォルトパターン読込み
		this.doPtnSet(this.ptnDefNum);

		//this.dispTimer.setTimeout(function() {
		//	aaoh.sora.App.main.setFocus();
		//}, 200);
	}

	/**
	 * 表示データの初期化。
	 *
	 * @param {String} type 表示タイプ
	 */
	lifegame.Main.prototype.
	initToolbarData = function(type) {
		var btnMax    = aaoh.kaze.Gadget.$('btnMax');
		var btnOption = aaoh.kaze.Gadget.$('btnOption');
		var btnClose  = aaoh.kaze.Gadget.$('btnClose');
		if (btnMax == null || btnOption == null || btnClose == null) {
			return;
		}

		var srcBtnMaxUp      = null;
		var srcBtnMaxOver    = null;
		var srcBtnMaxDown    = null;
		var srcBtnOptionUp   = null;
		var srcBtnOptionOver = null;
		var srcBtnOptionDown = null;
		var srcBtnCloseUp    = null;
		var srcBtnCloseOver  = null;
		var srcBtnCloseDown  = null;
		if (!aaoh.kaze.Type.isGoogle() && !aaoh.kaze.Type.isIE()) {
			srcBtnMaxUp    = 'images/lifegame' + type + '-max.png';
			srcBtnOptionUp = 'images/lifegame' + type + '-option.png';
			srcBtnCloseUp  = 'images/lifegame' + type + '-close.png';

		} else {
			srcBtnMaxUp    = 'sub-images/lifegame' + type + '-max_up.png';
			srcBtnOptionUp = 'sub-images/lifegame' + type + '-option_up.png';
			srcBtnCloseUp  = 'sub-images/lifegame' + type + '-close_up.png';
		}
		srcBtnMaxOver    = 'sub-images/lifegame' + type + '-max_over.png';
		srcBtnMaxDown    = 'sub-images/lifegame' + type + '-max_down.png';
		srcBtnOptionOver = 'sub-images/lifegame' + type + '-option_over.png';
		srcBtnOptionDown = 'sub-images/lifegame' + type + '-option_down.png';
		srcBtnCloseOver  = 'sub-images/lifegame' + type + '-close_over.png';
		srcBtnCloseDown  = 'sub-images/lifegame' + type + '-close_down.png';

		aaoh.kaze.Element.setBackgroundImage(
			btnMax, srcBtnMaxUp, srcBtnMaxOver, srcBtnMaxDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnOption, srcBtnOptionUp, srcBtnOptionOver, srcBtnOptionDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnClose, srcBtnCloseUp, srcBtnCloseOver, srcBtnCloseDown
		);
	}

	/**
	 * 表示データの初期化。
	 *
	 * @param {String} type 表示タイプ
	 */
	lifegame.Main.prototype.
	initOnDockCell = function(type) {
		var cells = this.getOnDockCells();
		this.clearCells(cells);

		if (aaoh.kaze.Type.isGoogle() || aaoh.kaze.Type.isYahoo()) {
				var ground = aaoh.kaze.Gadget.$('ground');
				aaoh.kaze.Element.setBackground(ground, this.borderColor);

		} else {
			for (var cnt = 0; cnt < cells.length; cnt++) {
				var cell = cells[cnt];

				aaoh.kaze.Element.setBorderColor(cell, this.borderColor);
			}
		} // if (aaoh.kaze.Type.isGoogle() || aaoh.kaze.Type.isYahoo()) 
	}

	/**
	 * 表示データの初期化。
	 *
	 * @param {String} type 表示タイプ
	 */
	lifegame.Main.prototype.
	initUnDockCell = function(type) {
		var cells = this.getUnDockCells();
		this.clearCells(cells);

		if (aaoh.kaze.Type.isGoogle() || aaoh.kaze.Type.isYahoo()) {
				var ground = aaoh.kaze.Gadget.$('ground_u');
				aaoh.kaze.Element.setBackground(ground, this.borderColor);

		} else {
			for (var cnt = 0; cnt < cells.length; cnt++) {
				var cell = cells[cnt];

				aaoh.kaze.Element.setBorderColor(cell, this.borderColor);
			}
		} // if (aaoh.kaze.Type.isGoogle() || aaoh.kaze.Type.isYahoo()) 
	}

	/**
	 * 表示データの初期化。
	 *
	 * @param {String} type 表示タイプ
	 */
	lifegame.Main.prototype.
	initOnDockData = function(type) {
		var btnPtn      = aaoh.kaze.Gadget.$('btnPtn');
		var btnPtn_d    = aaoh.kaze.Gadget.$('btnPtn_d');
		var btnStart    = aaoh.kaze.Gadget.$('btnStart');
		var btnStop     = aaoh.kaze.Gadget.$('btnStop');
		var btnClear    = aaoh.kaze.Gadget.$('btnClear');
		var btnClear_d  = aaoh.kaze.Gadget.$('btnClear_d');
		var lblGenTtl   = aaoh.kaze.Gadget.$('lblGenTtl');
		var lblGenCnt   = aaoh.kaze.Gadget.$('lblGenCnt');
		var lblPtnTtl   = aaoh.kaze.Gadget.$('lblPtnTtl');
		var btnPtnLeft  = aaoh.kaze.Gadget.$('btnPtnLeft');
		var btnPtnRight = aaoh.kaze.Gadget.$('btnPtnRight');
		var btnPtnClose = aaoh.kaze.Gadget.$('btnPtnClose');
		var ptnSetBg    = aaoh.kaze.Gadget.$('ptnSetBg');

		var srcPtnUp        = null;
		var srcPtnOver      = null;
		var srcPtnDown      = null;
		var srcPtn_dUp      = null;
		var srcStartUp      = null;
		var srcStartOver    = null;
		var srcStartDown    = null;
		var srcStopUp       = null;
		var srcStopOver     = null;
		var srcStopDown     = null;
		var srcClearUp      = null;
		var srcClearOver    = null;
		var srcClearDown    = null;
		var srcClear_dUp    = null;
		var srcPtnLeftUp    = null;
		var srcPtnLeftOver  = null;
		var srcPtnLeftDown  = null;
		var srcPtnRightUp   = null;
		var srcPtnRightOver = null;
		var srcPtnRightDown = null;
		var srcPtnSetBg     = 'images/bg-ptnset-bg' + type + '.png';

		if (!aaoh.kaze.Type.isGoogle() && !aaoh.kaze.Type.isIE()) {
			srcPtnUp      = 'images/lifegame' + type + '-ptn.png';
			srcStartUp    = 'images/lifegame' + type + '-start.png';
			srcStopUp     = 'images/lifegame' + type + '-stop.png';
			srcClearUp    = 'images/lifegame' + type + '-clear.png';
			srcPtnCloseUp = 'images/lifegame1-close.png';
			srcPtnLeftUp  = 'images/icon-allow-left.png';
			srcPtnRightUp = 'images/icon-allow-right.png';

		} else {
			srcPtnUp      = 'sub-images/lifegame' + type + '-ptn_up.png';
			srcStartUp    = 'sub-images/lifegame' + type + '-start_up.png';
			srcStopUp     = 'sub-images/lifegame' + type + '-stop_up.png';
			srcClearUp    = 'sub-images/lifegame' + type + '-clear_up.png';
			srcPtnCloseUp = 'sub-images/lifegame1-close_up.png';
			srcPtnLeftUp  = 'sub-images/icon-allow-left_up.png';
			srcPtnRightUp = 'sub-images/icon-allow-right_up.png';
		}
		srcPtn_dUp      = 'images/lifegame' + type + '-ptn_disabled.png';
		srcClear_dUp    = 'images/lifegame' + type + '-clear_disabled.png';

		srcPtnOver      = 'sub-images/lifegame' + type + '-ptn_over.png';
		srcPtnDown      = 'sub-images/lifegame' + type + '-ptn_down.png';
		srcStartOver    = 'sub-images/lifegame' + type + '-start_over.png';
		srcStartDown    = 'sub-images/lifegame' + type + '-start_down.png';
		srcStopOver     = 'sub-images/lifegame' + type + '-stop_over.png';
		srcStopDown     = 'sub-images/lifegame' + type + '-stop_down.png';
		srcClearOver    = 'sub-images/lifegame' + type + '-clear_over.png';
		srcClearDown    = 'sub-images/lifegame' + type + '-clear_down.png';
		srcPtnCloseOver = 'sub-images/lifegame1-close_over.png';
		srcPtnCloseDown = 'sub-images/lifegame1-close_down.png';
		srcPtnLeftOver  = 'sub-images/icon-allow-left_over.png';
		srcPtnLeftDown  = 'sub-images/icon-allow-left_down.png';
		srcPtnRightOver = 'sub-images/icon-allow-right_over.png';
		srcPtnRightDown = 'sub-images/icon-allow-right_down.png';

		aaoh.kaze.Element.setBackgroundImage(
			btnPtn, srcPtnUp, srcPtnOver, srcPtnDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtn_d, srcPtn_dUp
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnStart, srcStartUp, srcStartOver, srcStartDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnStop, srcStopUp, srcStopOver, srcStopDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnClear, srcClearUp, srcClearOver, srcClearDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnClear_d, srcClear_dUp
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtnClose, srcPtnCloseUp, srcPtnCloseOver, srcPtnCloseDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtnLeft, srcPtnLeftUp, srcPtnLeftOver, srcPtnLeftDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtnRight, srcPtnRightUp, srcPtnRightOver, srcPtnRightDown
		);

		if (!aaoh.kaze.Type.isGoogle()) {
			aaoh.kaze.Element.setBackgroundImage(ptnSetBg, srcPtnSetBg);

		} else {
			aaoh.kaze.Element.setImage(ptnSetBg, srcPtnSetBg);
		}

		if (!aaoh.kaze.Type.isYahoo()) {
			if (!!ptnSetBg.style && ptnSetBg.style.backgroundRepeat) {
				ptnSetBg.style.backgroundRepeat = 'repeat';
			}
			aaoh.kaze.Element.setOpacity(ptnSetBg, this.OPACITY_PTN_SET_BG);
		}

		var tipPtn      = aaoh.sora.App.control.locale.getString('TIP_PTN');
		var tipStart    = aaoh.sora.App.control.locale.getString('TIP_START');
		var tipStop     = aaoh.sora.App.control.locale.getString('TIP_STOP');
		var tipClear    = aaoh.sora.App.control.locale.getString('TIP_CLEAR');
		var ttlGen      = aaoh.sora.App.control.locale.getString('TTL_GEN');
		var ttlPtn      = aaoh.sora.App.control.locale.getString('TTL_PTN');
		var tipPtnClose = aaoh.sora.App.control.locale.getString('TIP_PTN_CLOSE');
		var tipPtnLeft  = aaoh.sora.App.control.locale.getString('TIP_PTN_LEFT');
		var tipPtnRight = aaoh.sora.App.control.locale.getString('TIP_PTN_RIGHT');

		aaoh.kaze.Element.setTooltip(btnPtn,      tipPtn);
		aaoh.kaze.Element.setTooltip(btnStart,    tipStart);
		aaoh.kaze.Element.setTooltip(btnStop,     tipStop);
		aaoh.kaze.Element.setTooltip(btnClear,    tipClear);
		aaoh.kaze.Element.setValue(lblGenTtl,     ttlGen);
		aaoh.kaze.Element.setValue(lblPtnTtl,     ttlPtn);
		aaoh.kaze.Element.setTooltip(btnPtnClose, tipPtnClose);
		aaoh.kaze.Element.setTooltip(btnPtnLeft,  tipPtnLeft);
		aaoh.kaze.Element.setTooltip(btnPtnRight, tipPtnRight);

		aaoh.kaze.Element.setForeground(lblGenTtl, this.bgOnColor);
		aaoh.kaze.Element.setForeground(lblGenCnt, this.bgOnColor);
	}

	/**
	 * 表示データの初期化。
	 *
	 * @param {String} type 表示タイプ
	 */
	lifegame.Main.prototype.
	initUnDockData = function(type) {
		var btnPtn      = aaoh.kaze.Gadget.$('btnPtn_u');
		var btnPtn_d    = aaoh.kaze.Gadget.$('btnPtn_d_u');
		var btnStart    = aaoh.kaze.Gadget.$('btnStart_u');
		var btnStop     = aaoh.kaze.Gadget.$('btnStop_u');
		var btnClear    = aaoh.kaze.Gadget.$('btnClear_u');
		var btnClear_d  = aaoh.kaze.Gadget.$('btnClear_d_u');
		var lblGenTtl   = aaoh.kaze.Gadget.$('lblGenTtl_u');
		var lblGenCnt   = aaoh.kaze.Gadget.$('lblGenCnt_u');
		var lblPtnTtl   = aaoh.kaze.Gadget.$('lblPtnTtl_u');
		var btnPtnLeft  = aaoh.kaze.Gadget.$('btnPtnLeft_u');
		var btnPtnRight = aaoh.kaze.Gadget.$('btnPtnRight_u');
		var btnPtnClose = aaoh.kaze.Gadget.$('btnPtnClose_u');
		var ptnSetBg    = aaoh.kaze.Gadget.$('ptnSetBg_u');

		var srcPtnUp        = null;
		var srcPtnOver      = null;
		var srcPtnDown      = null;
		var srcPtn_dUp      = null;
		var srcStartUp      = null;
		var srcStartOver    = null;
		var srcStartDown    = null;
		var srcStopUp       = null;
		var srcStopOver     = null;
		var srcStopDown     = null;
		var srcClearUp      = null;
		var srcClearOver    = null;
		var srcClearDown    = null;
		var srcClear_dUp    = null;
		var srcPtnLeftUp    = null;
		var srcPtnLeftOver  = null;
		var srcPtnLeftDown  = null;
		var srcPtnRightUp   = null;
		var srcPtnRightOver = null;
		var srcPtnRightDown = null;
		var srcPtnSetBg     = 'images/bg-ptnset-bg' + type + '.png';

		if (!aaoh.kaze.Type.isGoogle() && !aaoh.kaze.Type.isIE()) {
			srcPtnUp      = 'images/lifegame' + type + '-ptn_u.png';
			srcStartUp    = 'images/lifegame' + type + '-start_u.png';
			srcStopUp     = 'images/lifegame' + type + '-stop_u.png';
			srcClearUp    = 'images/lifegame' + type + '-clear_u.png';
			srcPtnCloseUp = 'images/lifegame1-close.png';
			srcPtnLeftUp  = 'images/icon-allow-left.png';
			srcPtnRightUp = 'images/icon-allow-right.png';

		} else {
			srcPtnUp      = 'sub-images/lifegame' + type + '-ptn_u_up.png';
			srcStartUp    = 'sub-images/lifegame' + type + '-start_u_up.png';
			srcStopUp     = 'sub-images/lifegame' + type + '-stop_u_up.png';
			srcClearUp    = 'sub-images/lifegame' + type + '-clear_u_up.png';
			srcPtnCloseUp = 'sub-images/lifegame1-close_up.png';
			srcPtnLeftUp  = 'sub-images/icon-allow-left_up.png';
			srcPtnRightUp = 'sub-images/icon-allow-right_up.png';
		}
		srcPtn_dUp      = 'images/lifegame' + type + '-ptn_u_disabled.png';
		srcClear_dUp    = 'images/lifegame' + type + '-clear_u_disabled.png';

		srcPtnOver      = 'sub-images/lifegame' + type + '-ptn_u_over.png';
		srcPtnDown      = 'sub-images/lifegame' + type + '-ptn_u_down.png';
		srcStartOver    = 'sub-images/lifegame' + type + '-start_u_over.png';
		srcStartDown    = 'sub-images/lifegame' + type + '-start_u_down.png';
		srcStopOver     = 'sub-images/lifegame' + type + '-stop_u_over.png';
		srcStopDown     = 'sub-images/lifegame' + type + '-stop_u_down.png';
		srcClearOver    = 'sub-images/lifegame' + type + '-clear_u_over.png';
		srcClearDown    = 'sub-images/lifegame' + type + '-clear_u_down.png';
		srcPtnCloseOver = 'sub-images/lifegame1-close_over.png';
		srcPtnCloseDown = 'sub-images/lifegame1-close_down.png';
		srcPtnLeftOver  = 'sub-images/icon-allow-left_over.png';
		srcPtnLeftDown  = 'sub-images/icon-allow-left_down.png';
		srcPtnRightOver = 'sub-images/icon-allow-right_over.png';
		srcPtnRightDown = 'sub-images/icon-allow-right_down.png';

		aaoh.kaze.Element.setBackgroundImage(
			btnPtn, srcPtnUp, srcPtnOver, srcPtnDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtn_d, srcPtn_dUp
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnStart, srcStartUp, srcStartOver, srcStartDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnStop, srcStopUp, srcStopOver, srcStopDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnClear, srcClearUp, srcClearOver, srcClearDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnClear_d, srcClear_dUp
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtnClose, srcPtnCloseUp, srcPtnCloseOver, srcPtnCloseDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtnLeft, srcPtnLeftUp, srcPtnLeftOver, srcPtnLeftDown
		);
		aaoh.kaze.Element.setBackgroundImage(
			btnPtnRight, srcPtnRightUp, srcPtnRightOver, srcPtnRightDown
		);

		if (!aaoh.kaze.Type.isGoogle()) {
			aaoh.kaze.Element.setBackgroundImage(ptnSetBg, srcPtnSetBg);

		} else {
			aaoh.kaze.Element.setImage(ptnSetBg, srcPtnSetBg);
		}

		if (!aaoh.kaze.Type.isYahoo()) {
			if (!!ptnSetBg.style && ptnSetBg.style.backgroundRepeat) {
				ptnSetBg.style.backgroundRepeat = 'repeat';
			}
			aaoh.kaze.Element.setOpacity(ptnSetBg, this.OPACITY_PTN_SET_BG);
		}

		var tipPtn      = aaoh.sora.App.control.locale.getString('TIP_PTN');
		var tipStart    = aaoh.sora.App.control.locale.getString('TIP_START');
		var tipStop     = aaoh.sora.App.control.locale.getString('TIP_STOP');
		var tipClear    = aaoh.sora.App.control.locale.getString('TIP_CLEAR');
		var ttlGen      = aaoh.sora.App.control.locale.getString('TTL_GEN');
		var ttlPtn      = aaoh.sora.App.control.locale.getString('TTL_PTN');
		var tipPtnClose = aaoh.sora.App.control.locale.getString('TIP_PTN_CLOSE');
		var tipPtnLeft  = aaoh.sora.App.control.locale.getString('TIP_PTN_LEFT');
		var tipPtnRight = aaoh.sora.App.control.locale.getString('TIP_PTN_RIGHT');

		aaoh.kaze.Element.setTooltip(btnPtn,      tipPtn);
		aaoh.kaze.Element.setTooltip(btnStart,    tipStart);
		aaoh.kaze.Element.setTooltip(btnStop,     tipStop);
		aaoh.kaze.Element.setTooltip(btnClear,    tipClear);
		aaoh.kaze.Element.setValue(lblGenTtl,     ttlGen);
		aaoh.kaze.Element.setValue(lblPtnTtl,     ttlPtn);
		aaoh.kaze.Element.setTooltip(btnPtnClose, tipPtnClose);
		aaoh.kaze.Element.setTooltip(btnPtnLeft,  tipPtnLeft);
		aaoh.kaze.Element.setTooltip(btnPtnRight, tipPtnRight);

		aaoh.kaze.Element.setForeground(lblGenTtl, this.bgOnColor);
		aaoh.kaze.Element.setForeground(lblGenCnt, this.bgOnColor);
	}

	/**
	 * イベントの設定。
	 */
	lifegame.Main.prototype.
	setEventListener = function() {
		this.setLifegameEventListener();
		this.setOnDockEventListener();
		this.setUnDockEventListener();
	}

	/**
	 * イベントの設定。
	 */
	lifegame.Main.prototype.
	setLifegameEventListener = function() {
		// 特になし
	}

	/**
	 * イベントの設定。
	 */
	lifegame.Main.prototype.
	setOnDockEventListener = function() {
		var btnPtn = aaoh.kaze.Gadget.$('btnPtn');
		if (btnPtn != null) {
			aaoh.kaze.Element.setOnClick(btnPtn, function() {
				var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm');
				var ptnNo    = aaoh.sora.App.main.getNowPtnNo(null)
				aaoh.sora.App.main.setImgPtnNm(imgPtnNm, ptnNo);

				var ptnSetBg = aaoh.sora.App.main.getPtnSetBg();
				var ptnSet   = aaoh.sora.App.main.getPtnSet();
				aaoh.sora.App.main.setToggleButton(ptnSetBg, null);
				aaoh.sora.App.main.setFadeToggleButton(ptnSet, null);
				aaoh.sora.App.main.setFocus();

				aaoh.sora.App.main.flgPtnSet = true;
			});
		}

		var btnStart = aaoh.kaze.Gadget.$('btnStart');
		if (btnStart != null) {
			aaoh.kaze.Element.setOnClick(btnStart, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				aaoh.sora.App.main.doStart();
				aaoh.sora.App.main.setFocus();
			});
		}

		var btnStop = aaoh.kaze.Gadget.$('btnStop');
		if (btnStop != null) {
			aaoh.kaze.Element.setOnClick(btnStop, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				aaoh.sora.App.main.doStop(false);
				aaoh.sora.App.main.setFocus();
			});
		}

		var btnClear = aaoh.kaze.Gadget.$('btnClear');
		if (btnClear != null) {
			aaoh.kaze.Element.setOnClick(btnClear, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				var cells = aaoh.sora.App.main.getOnDockCells();
				aaoh.sora.App.main.clearCells(cells);
				aaoh.sora.App.main.setFocus();
			});
		}

		var btnPtnLeft = aaoh.kaze.Gadget.$('btnPtnLeft');
		if (btnPtnLeft != null) {
			aaoh.kaze.Element.setOnClick(btnPtnLeft, function() {
				var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm');
				var ptnNo    = aaoh.sora.App.main.getPreviousPtnNo(imgPtnNm)

				aaoh.sora.App.main.setImgPtnNm(imgPtnNm, ptnNo);
			});
		}

		var btnPtnRight = aaoh.kaze.Gadget.$('btnPtnRight');
		if (btnPtnRight != null) {
			aaoh.kaze.Element.setOnClick(btnPtnRight, function() {
				var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm');
				var ptnNo    = aaoh.sora.App.main.getNextPtnNo(imgPtnNm)

				aaoh.sora.App.main.setImgPtnNm(imgPtnNm, ptnNo);
			});
		}

		var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm');
		if (imgPtnNm != null) {
			aaoh.kaze.Element.setOnClick(imgPtnNm, function() {
				var cells = aaoh.sora.App.main.getOnDockCells();
				aaoh.sora.App.main.clearCells(cells);

				var ptnNo = aaoh.sora.App.main.getNowPtnNo(imgPtnNm)
				aaoh.sora.App.main.doPtnSet(ptnNo);

				var ptnSetBg = aaoh.sora.App.main.getPtnSetBg();
				var ptnSet   = aaoh.sora.App.main.getPtnSet();
				aaoh.sora.App.main.setToggleButton(null, ptnSetBg);
				aaoh.sora.App.main.setFadeToggleButton(null, ptnSet);
				aaoh.sora.App.main.setFocus();

				aaoh.sora.App.main.ptnDefNum = ptnNo;
				aaoh.sora.App.main.flgPtnSet = false;
			});
		}

		var btnPtnClose = aaoh.kaze.Gadget.$('btnPtnClose');
		if (btnPtnClose != null) {
			aaoh.kaze.Element.setOnClick(btnPtnClose, function() {
				var ptnSetBg = aaoh.sora.App.main.getPtnSetBg();
				var ptnSet   = aaoh.sora.App.main.getPtnSet();
				aaoh.sora.App.main.setToggleButton(null, ptnSetBg);
				aaoh.sora.App.main.setFadeToggleButton(null, ptnSet);
				aaoh.sora.App.main.setFocus();

				aaoh.sora.App.main.flgPtnSet = false;
			});
		}

		var cells = this.getOnDockCells();
		for (var cnt = 0; cnt < cells.length; cnt++) {
			var cell = cells[cnt];
			if (cell == null) {
				continue;
			}

			aaoh.kaze.Element.setOnClick(cell, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				// google対応
				var tmpCell = null;
				if (aaoh.kaze.Type.isGoogle()) {
					tmpCell = view.event.srcElement;
				} else {
					tmpCell = this;
				}

				aaoh.sora.App.main.setToggleCell(tmpCell);
			});
		}
	}

	/**
	 * イベントの設定。
	 */
	lifegame.Main.prototype.
	setUnDockEventListener = function() {
		var btnPtn = aaoh.kaze.Gadget.$('btnPtn_u');
		if (btnPtn != null) {
			aaoh.kaze.Element.setOnClick(btnPtn, function() {
				var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm_u');
				var ptnNo    = aaoh.sora.App.main.getNowPtnNo(null)
				aaoh.sora.App.main.setImgPtnNm(imgPtnNm, ptnNo);

				var ptnSetBg = aaoh.sora.App.main.getPtnSetBg();
				var ptnSet   = aaoh.sora.App.main.getPtnSet();
				aaoh.sora.App.main.setToggleButton(ptnSetBg, null);
				aaoh.sora.App.main.setFadeToggleButton(ptnSet, null);

				aaoh.sora.App.main.flgPtnSet = true;
			});
		}

		var btnStart = aaoh.kaze.Gadget.$('btnStart_u');
		if (btnStart != null) {
			aaoh.kaze.Element.setOnClick(btnStart, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				aaoh.sora.App.main.doStart();
				aaoh.sora.App.main.setFocus();
			});
		}

		var btnStop = aaoh.kaze.Gadget.$('btnStop_u');
		if (btnStop != null) {
			aaoh.kaze.Element.setOnClick(btnStop, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				aaoh.sora.App.main.doStop(false);
				aaoh.sora.App.main.setFocus();
			});
		}

		var btnClear = aaoh.kaze.Gadget.$('btnClear_u');
		if (btnClear != null) {
			aaoh.kaze.Element.setOnClick(btnClear, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				var cells = aaoh.sora.App.main.getUnDockCells();
				aaoh.sora.App.main.clearCells(cells);
				aaoh.sora.App.main.setFocus();
			});
		}

		var btnPtnLeft = aaoh.kaze.Gadget.$('btnPtnLeft_u');
		if (btnPtnLeft != null) {
			aaoh.kaze.Element.setOnClick(btnPtnLeft, function() {
				var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm_u');
				var ptnNo    = aaoh.sora.App.main.getPreviousPtnNo(imgPtnNm)

				aaoh.sora.App.main.setImgPtnNm(imgPtnNm, ptnNo);
			});
		}

		var btnPtnRight = aaoh.kaze.Gadget.$('btnPtnRight_u');
		if (btnPtnRight != null) {
			aaoh.kaze.Element.setOnClick(btnPtnRight, function() {
				var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm_u');
				var ptnNo    = aaoh.sora.App.main.getNextPtnNo(imgPtnNm)

				aaoh.sora.App.main.setImgPtnNm(imgPtnNm, ptnNo);
			});
		}

		var imgPtnNm = aaoh.kaze.Gadget.$('imgPtnNm_u');
		if (imgPtnNm != null) {
			aaoh.kaze.Element.setOnClick(imgPtnNm, function() {
				var cells = aaoh.sora.App.main.getUnDockCells();
				aaoh.sora.App.main.clearCells(cells);

				var ptnNo = aaoh.sora.App.main.getNowPtnNo(imgPtnNm)
				aaoh.sora.App.main.doPtnSet(ptnNo);

				var ptnSetBg = aaoh.sora.App.main.getPtnSetBg();
				var ptnSet   = aaoh.sora.App.main.getPtnSet();
				aaoh.sora.App.main.setToggleButton(null, ptnSetBg);
				aaoh.sora.App.main.setFadeToggleButton(null, ptnSet);
				aaoh.sora.App.main.setFocus();

				aaoh.sora.App.main.ptnDefNum = ptnNo;
				aaoh.sora.App.main.flgPtnSet = false;
			});
		}

		var btnPtnClose = aaoh.kaze.Gadget.$('btnPtnClose_u');
		if (btnPtnClose != null) {
			aaoh.kaze.Element.setOnClick(btnPtnClose, function() {
				var ptnSetBg = aaoh.sora.App.main.getPtnSetBg();
				var ptnSet   = aaoh.sora.App.main.getPtnSet();
				aaoh.sora.App.main.setToggleButton(null, ptnSetBg);
				aaoh.sora.App.main.setFadeToggleButton(null, ptnSet);
				aaoh.sora.App.main.setFocus();

				aaoh.sora.App.main.flgPtnSet = false;
			});
		}

		var cells = this.getUnDockCells();
		for (var cnt = 0; cnt < cells.length; cnt++) {
			var cell = cells[cnt];
			if (cell == null) {
				continue;
			}

			aaoh.kaze.Element.setOnClick(cell, function() {
				if (aaoh.sora.App.main.flgPtnSet) {
					return;
				}

				// google対応
				var tmpCell = null;
				if (aaoh.kaze.Type.isGoogle()) {
					tmpCell = view.event.srcElement;
				} else {
					tmpCell = this;
				}

				aaoh.sora.App.main.setToggleCell(tmpCell);
			});
		}
	}

	/**
	 * 初期化処理。
	 */
	lifegame.Main.prototype.
	init = function() {
		this.setEventListener();
	}
})();
